@@ -406,12 +406,7 @@ fn launch_libmdns(
             }
             .map_err(|e| DiscoveryError::DnsSdError(Box::new(e)))?;
 
-            let svc = responder.register(
-                DNS_SD_SERVICE_NAME.to_owned(),
-                name.into_owned(),
-                port,
-                &TXT_RECORD,
-            );
+            let svc = responder.register(DNS_SD_SERVICE_NAME, &name, port, &TXT_RECORD);
 
             let _ = shutdown_rx.blocking_recv();
 
@@ -442,6 +437,7 @@ impl Builder {
                 is_group: false,
                 device_id: device_id.into(),
                 client_id: client_id.into(),
+                aliases: Vec::new(),
             },
             port: 0,
             zeroconf_ip: vec![],
@@ -467,6 +463,21 @@ impl Builder {
         self
     }
 
+    /// Adds an alias for this device. Multiple aliases can be added by calling this method multiple times.
+    pub fn add_alias(
+        mut self,
+        alias: impl Into<Cow<'static, str>>,
+        id: u32,
+        is_group: bool,
+    ) -> Self {
+        self.server_config.aliases.push(server::Alias {
+            name: alias.into(),
+            id,
+            is_group,
+        });
+        self
+    }
+
     /// Set the ip addresses on which it should listen to incoming connections. The default is all interfaces.
     pub fn zeroconf_ip(mut self, zeroconf_ip: Vec<std::net::IpAddr>) -> Self {
         self.zeroconf_ip = zeroconf_ip;
